@@ -1,4 +1,4 @@
-// Emacs style mode select   -*- C++ -*-
+// Emacs style mode select   -*- C -*-
 //-----------------------------------------------------------------------------
 //
 // Copyright(C) 1993-1996 Id Software, Inc.
@@ -1158,13 +1158,11 @@ static void ScheduleTrack(opl_track_data_t *track)
 {
     unsigned int nticks;
     unsigned int ms;
-    static int total = 0;
 
     // Get the number of milliseconds until the next event.
 
     nticks = MIDI_GetDeltaTime(track->iter);
     ms = (nticks * track->ms_per_beat) / track->ticks_per_beat;
-    total += ms;
 
     // Set a timer to be invoked when the next event is
     // ready to play.
@@ -1362,7 +1360,7 @@ static const void *I_OPL_RegisterSong(const void *data, unsigned len)
     // time numbers we have to traverse the tracks and everything
     if (mf.len < 100)
     {
-        lprintf (LO_WARN, "I_OPL_RegisterSong: Very short MIDI (%I64i bytes)\n", mf.len);
+        lprintf (LO_WARN, "I_OPL_RegisterSong: Very short MIDI (%llu bytes)\n", (unsigned long long)mf.len);
         return NULL;
     }
 
