@@ -1,4 +1,4 @@
-/* Emacs style mode select   -*- C++ -*-
+/* Emacs style mode select   -*- C -*-
  *-----------------------------------------------------------------------------
  *
  *
@@ -402,7 +402,7 @@ while (SDL_PollEvent(Event))
     break;
 
   case SDL_QUIT:
-    S_StartSound(NULL, sfx_swtchn);
+    S_StartVoidSound(sfx_swtchn);
     M_QuitDOOM(0);
 
   default:
@@ -758,7 +758,7 @@ static void I_FillScreenResolutionsList(void)
 {
   int display_index = 0;
   SDL_DisplayMode mode;
-  int i, j, list_size, current_resolution_index, count;
+  int i, list_size, current_resolution_index, count;
   char desired_resolution[256];
 
   // do it only once
@@ -788,8 +788,6 @@ static void I_FillScreenResolutionsList(void)
 
     for(i = count - 1 + num_canonicals; i >= 0; i--)
     {
-      int in_list = false;
-
       // make sure the canonical resolutions are always available
       if (i > count - 1)
       {
@@ -1194,7 +1192,6 @@ void I_UpdateVideoMode(void)
     if (V_IsOpenGLMode())
     {
       gld_CleanMemory();
-      // hires patches
       gld_CleanStaticMemory();
     }
 
@@ -1262,8 +1259,6 @@ void I_UpdateVideoMode(void)
       init_flags);
     sdl_glcontext = SDL_GL_CreateContext(sdl_window);
     SDL_SetWindowMinimumSize(sdl_window, SCREENWIDTH, SCREENHEIGHT);
-
-    gld_CheckHardwareGamma();
   }
   else
   {
@@ -1564,22 +1559,6 @@ static void UpdateFocus(void)
     V_SetPalette(st_palette);
   }
 
-  if (V_IsOpenGLMode())
-  {
-    if (gl_hardware_gamma)
-    {
-      if (!window_focused)
-      {
-        // e6y: Restore of startup gamma if window loses focus
-        gld_SetGammaRamp(-1);
-      }
-      else
-      {
-        gld_SetGammaRamp(gl_usegamma);
-      }
-    }
-  }
-
   // Should the screen be grabbed?
   //    screenvisible = (state & SDL_APPACTIVE) != 0;
 }
