@@ -69,7 +69,7 @@ static arg_config_t arg_config[dsda_arg_count] = {
     arg_string_array, AT_LEAST_ONE_STRING,
   },
   [dsda_arg_deh] = {
-    "-deh", NULL, NULL,
+    "-deh", "-bex", NULL,
     "loads additional deh files",
     arg_string_array, AT_LEAST_ONE_STRING,
   },
@@ -193,6 +193,11 @@ static arg_config_t arg_config[dsda_arg_count] = {
     "automatically pistol start each map",
     arg_null,
   },
+  [dsda_arg_chain_episodes] = {
+    "-chain_episodes", NULL, NULL,
+    "completing one episode leads to the next without interruption",
+    arg_null,
+  },
   [dsda_arg_stroller] = {
     "-stroller", NULL, NULL,
     "applies stroller category limitations",
@@ -203,6 +208,11 @@ static arg_config_t arg_config[dsda_arg_count] = {
     "sets player speed percent",
     arg_int, 10, 255,
   },
+  [dsda_arg_game_speed] = {
+    "-game_speed", NULL, NULL,
+    "sets game speed percent",
+    arg_int, 10, 10000,
+  },
   [dsda_arg_tas] = {
     "-tas", NULL, NULL,
     "lifts strict mode restrictions",
@@ -213,11 +223,21 @@ static arg_config_t arg_config[dsda_arg_count] = {
     "starts in build mode",
     arg_null,
   },
+  [dsda_arg_quit_after_brute_force] = {
+    "-quit_after_brute_force", NULL, NULL,
+    "quits the game when brute force ends",
+    arg_null,
+  },
   [dsda_arg_first_input] = {
     "-first_input", NULL, NULL,
     "builds the first frame F S T",
     arg_int_array, -128, 127, 3, 3
   },
+  [dsda_arg_command] = {
+    "-command", NULL, NULL,
+    "runs a console command",
+    arg_string
+  },
   [dsda_arg_skipsec] = {
     "-skipsec", NULL, NULL,
     "skip to the given time (mm:ss or ss) - negative times seek from the end",
@@ -403,9 +423,9 @@ static arg_config_t arg_config[dsda_arg_count] = {
     "force classic bsp nodes",
     arg_null,
   },
-  [dsda_arg_devparm] = {
-    "-devparm", NULL, NULL,
-    "turn on old development mode",
+  [dsda_arg_sigsegv] = {
+    "-sigsegv", NULL, NULL,
+    "disable the SIGSEGV signal handler",
     arg_null,
   },
   [dsda_arg_deathmatch] = {
