@@ -25,9 +25,11 @@
 
 extern menu_t dsda_ConsoleDef;
 
+int dsda_ConsoleHeight(void);
 dboolean dsda_OpenConsole(void);
 void dsda_UpdateConsoleText(char* text);
 void dsda_UpdateConsole(int action);
 void dsda_ExecuteConsoleScript(int i);
+void dsda_InterpretConsoleCommands(const char* str, dboolean noise);
 
 #endif
