@@ -22,6 +22,7 @@
 
 void dsda_RestartPlayback(void);
 dboolean dsda_JumpToLogicTic(int tic);
+dboolean dsda_JumpToLogicTicFrom(int tic, int from_tic);
 void dsda_ExecutePlaybackOptions(void);
 const char* dsda_ParsePlaybackOptions(void);
 const char* dsda_PlaybackName(void);
@@ -30,4 +31,5 @@ void dsda_AttachPlaybackStream(const byte* demo_p, int length, int behaviour);
 int dsda_PlaybackTics(void);
 void dsda_StorePlaybackPosition(void);
 void dsda_RestorePlaybackPosition(void);
+void dsda_JoinDemo(ticcmd_t* cmd);
 void dsda_TryPlaybackOneTick(ticcmd_t* cmd);
