@@ -1,4 +1,4 @@
-/* Emacs style mode select   -*- C++ -*-
+/* Emacs style mode select   -*- C -*-
  *-----------------------------------------------------------------------------
  *
  *
@@ -44,6 +44,7 @@
 #include "heretic/f_finale.h"
 #include "hexen/f_finale.h"
 
+#include "dsda/font.h"
 #include "dsda/mapinfo.h"
 
 #include "f_finale.h" // CPhipps - hmm...
@@ -358,8 +359,6 @@ void F_Ticker(void)
 // CPhipps - reformatted
 
 #include "hu_stuff.h"
-extern patchnum_t hu_font[HU_FONTSIZE];
-
 
 void F_TextWrite (void)
 {
@@ -370,6 +369,7 @@ void F_TextWrite (void)
   }
   else
     V_DrawBackground(finaleflat, 0);
+
   { // draw some of the text onto the screen
     int         cx = 10;
     int         cy = 10;
@@ -384,24 +384,26 @@ void F_TextWrite (void)
       int       c = *ch++;
 
       if (!c)
-  break;
+        break;
+
       if (c == '\n') {
-  cx = 10;
-  cy += 11;
-  continue;
+        cx = 10;
+        cy += 11;
+        continue;
       }
 
       c = toupper(c) - HU_FONTSTART;
       if (c < 0 || c> HU_FONTSIZE) {
-  cx += 4;
-  continue;
+        cx += 4;
+        continue;
       }
 
-      w = hu_font[c].width;
+      w = hud_font.font[c].width;
       if (cx+w > SCREENWIDTH)
-  break;
+        break;
+
       // CPhipps - patch drawing updated
-      V_DrawNumPatch(cx, cy, 0, hu_font[c].lumpnum, CR_DEFAULT, VPT_STRETCH);
+      V_DrawNumPatch(cx, cy, 0, hud_font.font[c].lumpnum, CR_DEFAULT, VPT_STRETCH);
       cx+=w;
     }
   }
@@ -486,7 +488,7 @@ void F_CastTicker (void)
     if (castorder[castnum].name == NULL)
       castnum = 0;
     if (mobjinfo[castorder[castnum].type].seesound)
-      S_StartSound (NULL, mobjinfo[castorder[castnum].type].seesound);
+      S_StartVoidSound(mobjinfo[castorder[castnum].type].seesound);
     caststate = &states[mobjinfo[castorder[castnum].type].seestate];
     castframes = 0;
   }
@@ -532,7 +534,7 @@ void F_CastTicker (void)
     }
 
     if (sfx)
-      S_StartSound (NULL, sfx);
+      S_StartVoidSound(sfx);
   }
 
   if (castframes == 12)
@@ -592,7 +594,7 @@ dboolean F_CastResponder (event_t* ev)
   castframes = 0;
   castattacking = false;
   if (mobjinfo[castorder[castnum].type].deathsound)
-    S_StartSound (NULL, mobjinfo[castorder[castnum].type].deathsound);
+    S_StartVoidSound(mobjinfo[castorder[castnum].type].deathsound);
 
   return true;
 }
@@ -622,7 +624,7 @@ static void F_CastPrint (const char* text) // CPhipps - static, const char*
       continue;
     }
 
-    w = hu_font[c].width;
+    w = hud_font.font[c].width;
     width += w;
   }
 
@@ -641,9 +643,9 @@ static void F_CastPrint (const char* text) // CPhipps - static, const char*
       continue;
     }
 
-    w = hu_font[c].width;
+    w = hud_font.font[c].width;
     // CPhipps - patch drawing updated
-    V_DrawNumPatch(cx, 180, 0, hu_font[c].lumpnum, CR_DEFAULT, VPT_STRETCH);
+    V_DrawNumPatch(cx, 180, 0, hud_font.font[c].lumpnum, CR_DEFAULT, VPT_STRETCH);
     cx+=w;
   }
 }
@@ -742,7 +744,7 @@ void F_BunnyScroll (void)
     stage = 6;
   if (stage > laststage)
   {
-    S_StartSound (NULL, sfx_pistol);
+    S_StartVoidSound(sfx_pistol);
     laststage = stage;
   }
 
