@@ -44,6 +44,7 @@
 #include "gl_opengl.h"
 
 #include <SDL.h>
+#include <math.h>
 
 #include "doomstat.h"
 #include "v_video.h"
@@ -86,12 +87,6 @@ static PalEntry_t *SkyColor;
 SkyBoxParams_t SkyBox;
 float y_offset_saved;
 
-// skybox
-box_skybox_t *BoxSkybox = NULL;
-int BoxSkyboxCount = 0;
-
-box_skybox_t *BoxSkybox_default;
-
 void gld_InitSky(void)
 {
   memset(&SkyBox, 0, sizeof(SkyBox));
@@ -103,11 +98,8 @@ void gld_InitFrameSky(void)
 {
   SkyBox.type = SKY_NONE;
   SkyBox.wall.gltexture = NULL;
-  SkyBox.x_scale = 0;
-  SkyBox.y_scale = 0;
   SkyBox.x_offset = 0;
   SkyBox.y_offset = 0;
-  SkyBox.side = NULL;
 }
 
 void gld_DrawFakeSkyStrips(void)
@@ -141,33 +133,13 @@ void gld_DrawFakeSkyStrips(void)
   glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
 }
 
-void gld_GetScreenSkyScale(GLWall *wall, float *scale_x, float *scale_y)
-{
-  float sx, sy;
-
-  sx = (wall->flag == GLDWF_SKYFLIP ? -128.0f : 128.0f);
-
-  if (!mlook_or_fov)
-  {
-    sx = sx / (float)wall->gltexture->buffer_width;
-    sy = 200.0f / (wall->gltexture->buffer_height * 1.25f);
-  }
-  else
-  {
-    sx = sx * skyscale / (float)wall->gltexture->buffer_width;
-    sy = 127.0f * skyscale / (wall->gltexture->buffer_height * 1.25f);
-  }
-
-  *scale_x = sx;
-  *scale_y = sy;
-}
-
 // Sky textures with a zero index should be forced
 // See third episode of requiem.wad
 void gld_AddSkyTexture(GLWall *wall, int sky1, int sky2, int skytype)
 {
   side_t *s = NULL;
   line_t *l = NULL;
+
   wall->gltexture = NULL;
 
   if ((sky1) & PL_SKYFLAT)
@@ -185,22 +157,16 @@ void gld_AddSkyTexture(GLWall *wall, int sky1, int sky2, int skytype)
   if (l)
   {
     s = *l->sidenum + sides;
-    SkyBox.side = s;
     wall->gltexture = gld_RegisterSkyTexture(texturetranslation[s->toptexture],
       texturetranslation[s->toptexture] == skytexture || l->special == 271 || l->special == 272);
     if (wall->gltexture)
     {
-      if (!mlook_or_fov)
-      {
-        wall->skyyaw  = -2.0f*((-(float)((viewangle+s->textureoffset)>>ANGLETOFINESHIFT)*360.0f/FINEANGLES)/90.0f);
-        wall->skyymid = 200.0f/319.5f*(((float)s->rowoffset/(float)FRACUNIT - 28.0f)/100.0f);
-      }
-      else
-      {
-        wall->skyyaw  = -2.0f*(((270.0f-(float)((viewangle+s->textureoffset)>>ANGLETOFINESHIFT)*360.0f/FINEANGLES)+90.0f)/90.0f/skyscale);
-        wall->skyymid = skyYShift+(((float)s->rowoffset/(float)FRACUNIT + 28.0f)/wall->gltexture->buffer_height)/skyscale;
-      }
-      wall->flag = (l->special == 272 ? GLDWF_SKY : GLDWF_SKYFLIP);
+      // As far as I can tell, textureoffset just adds directly to viewangle and is
+      // therefore affected by tiling, but rowoffset is not.
+      wall->skyyaw = (double) (viewangle + s->textureoffset) / (double) ANGLE_MAX;
+      wall->skypitch = skyYShift;
+      wall->skyoffset = (((float)s->rowoffset/(float)FRACUNIT - 28.0f)/wall->gltexture->buffer_height);
+      wall->flag = l->special == 282 ? GLDWF_SKY : GLDWF_SKYFLIP;
     }
   }
   else
@@ -208,8 +174,17 @@ void gld_AddSkyTexture(GLWall *wall, int sky1, int sky2, int skytype)
     wall->gltexture = gld_RegisterSkyTexture(skytexture, true);
     if (wall->gltexture)
     {
-      wall->skyyaw  = skyXShift;
-      wall->skyymid = skyYShift;
+      int h = wall->gltexture->buffer_height;
+
+      wall->skyyaw = skyXShift;
+      wall->skypitch = skyYShift;
+      // Choose offset based on logic from r_sky.c
+      if (h >= 128 && h < 200)
+        wall->skyoffset = -28.0f / 128.0f;
+      else if (h > 200)
+        wall->skyoffset = (200 - h) / 128.0f;
+      else
+        wall->skyoffset = 0.0f;
       wall->flag = GLDWF_SKY;
     }
   }
@@ -228,58 +203,87 @@ void gld_AddSkyTexture(GLWall *wall, int sky1, int sky2, int skytype)
 
       switch (gl_drawskys)
       {
-      case skytype_standard:
-        gld_GetScreenSkyScale(wall, &SkyBox.x_scale, &SkyBox.y_scale);
-        break;
-
-      case skytype_screen:
-        if (s)
-        {
-          SkyBox.x_offset = (float)s->textureoffset;
-          SkyBox.y_offset = (float)s->rowoffset / (float)FRACUNIT;
-        }
-        break;
       case skytype_skydome:
         if (s)
         {
           SkyBox.x_offset = (float)s->textureoffset * 180.0f / (float)ANG180;
           SkyBox.y_offset = (float)s->rowoffset / (float)FRACUNIT;
         }
+        else if (raven)
+        {
+          SkyBox.y_offset = 39.0f;
+        }
         break;
       }
     }
   }
 }
 
+// The fussy arithmetic to correctly scale and translate the sky texture lives here.
+void gld_SkyTransform(GLWall* wall)
+{
+  // Make apparent scale of sky closer to software
+  const float scale_correction = 0.80f;
+  // 360 horizontal degrees of sky in texels (4 tilings of default sky)
+  float sky_width = 256 * 4;
+  // 360 vertical degrees of sky in texels (determined emperically with Heretic
+  // software mode)
+  float sky_height = 880;
+  // Texture dimensions
+  float w = wall->gltexture->buffer_width;
+  float h = wall->gltexture->buffer_height;
+  // Tile factors
+  float tilex = sky_width / w;
+  float tiley = sky_height / h;
+  // Adjustment for tall screens
+  float ratio = tallscreen ? (float)ratio_multiplier / ratio_scale : 1.0f;
+  // X flip coefficient
+  float flipx = wall->flag == GLDWF_SKYFLIP ? -1.0 : 1.0;
+  // Scale factors
+  float scalex = scale_correction / skyscale * flipx;
+  float scaley = scale_correction * ratio * (skystretch ? 2.0 : 1.0f) / skyscale;
+  // Translations
+  float transx = wall->skyyaw * tilex * flipx;
+  float transy = wall->skypitch * tiley;
+
+  // Apply scaling centered at horizon and view yaw
+  glTranslatef(transx, wall->skyoffset, 0.0f);
+  glScalef(1.0f / scalex, 1.0f / scaley, 1.0);
+  //glTranslatef(-transx, -wall->skyoffset, 0.0f);
+
+  // Apply offset
+  //glTranslatef(0, wall->skyoffset, 0.0f);
+
+  // Apply view yaw and pitch
+  //glTranslatef(transx, transy, 0.0f);
+
+  // The above translations (which are part of logically distinct operations)
+  // are commented out and combined here for efficiency
+  glTranslatef(0.0, transy, 0.0f);
+
+  // Scale texture coordinates generated by glTexGen so the range [0, 1]
+  // corresponds to the full range of the texture as usual
+  glScalef(MAP_COEFF / w, MAP_COEFF / h, 1.0f);
+}
+
 void gld_DrawStripsSky(void)
 {
   int i;
-  float skyymid_multiplier;
   GLTexture *gltexture = NULL;
 
   if (gl_drawskys == skytype_standard)
   {
-    if (comp[comp_skymap] && (invul_method & INVUL_BW))
-      glTexEnvi(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
-
     glEnable(GL_TEXTURE_GEN_S);
     glEnable(GL_TEXTURE_GEN_T);
     glEnable(GL_TEXTURE_GEN_Q);
-    if (comp[comp_skymap] || !(invul_method & INVUL_BW))
-      glColor4fv(gl_whitecolor);
+
+    glColor4fv(gl_whitecolor);
 
     SetTextureMode(TM_OPAQUE);
   }
 
-  gld_EnableDetail(false);
   glMatrixMode(GL_TEXTURE);
 
-  skyymid_multiplier = 1.0f;
-  if (tallscreen)
-  {
-    skyymid_multiplier = (float)ratio_multiplier / ratio_scale;
-  }
-
   for (i = gld_drawinfo.num_items[GLDIT_SWALL] - 1; i >= 0; i--)
   {
     GLWall *wall = gld_drawinfo.items[GLDIT_SWALL][i].item.wall;
@@ -294,13 +298,9 @@ void gld_DrawStripsSky(void)
 
     if (gltexture)
     {
-      float sx, sy;
-
       glPushMatrix();
 
-      gld_GetScreenSkyScale(wall, &sx, &sy);
-      glScalef(sx, sy * skyymid_multiplier, 1.0f);
-      glTranslatef(wall->skyyaw, wall->skyymid / skyymid_multiplier, 0.0f);
+      gld_SkyTransform(wall);
     }
 
     glBegin(GL_TRIANGLE_STRIP);
@@ -326,9 +326,6 @@ void gld_DrawStripsSky(void)
     glDisable(GL_TEXTURE_GEN_T);
     glDisable(GL_TEXTURE_GEN_S);
 
-    if (comp[comp_skymap] && (invul_method & INVUL_BW))
-      glTexEnvi(GL_TEXTURE_ENV,GL_COMBINE_RGB,GL_COMBINE);
-
     SetFrameTextureMode();
   }
 }
@@ -339,13 +336,12 @@ void gld_DrawSkyCaps(void)
   {
     if (dsda_MouseLook())
     {
-      gld_BindTexture(SkyBox.wall.gltexture, 0);
+      gld_BindSkyTexture(SkyBox.wall.gltexture);
 
       glMatrixMode(GL_TEXTURE);
       glPushMatrix();
 
-      glScalef(SkyBox.x_scale, SkyBox.y_scale, 1.0f);
-      glTranslatef(SkyBox.wall.skyyaw, SkyBox.wall.skyymid, 0.0f);
+      gld_SkyTransform(&SkyBox.wall);
 
       if (SkyBox.type & SKY_CEILING)
       {
@@ -428,90 +424,6 @@ void averageColor(PalEntry_t * PalEntry, const unsigned int *data, int size, fix
   return;
 }
 
-// It is an alternative way of drawing the sky (gl_drawskys == skytype_screen)
-// This method make sense only for old hardware which have no support for GL_TEXTURE_GEN_*
-// Voodoo as example
-void gld_DrawScreenSkybox(void)
-{
-  if (SkyBox.wall.gltexture)
-  {
-    #define WRAPANGLE (ANGLE_MAX/4)
-
-    float fU1, fU2, fV1, fV2;
-    GLWall *wall = &SkyBox.wall;
-    angle_t angle;
-    int i, k;
-    float w;
-
-    glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE); // no graphics
-    gld_EnableTexture2D(GL_TEXTURE0_ARB, false);
-
-    for (i = gld_drawinfo.num_items[GLDIT_SWALL] - 1; i >= 0; i--)
-    {
-      GLWall* wall = gld_drawinfo.items[GLDIT_SWALL][i].item.wall;
-
-      glBegin(GL_TRIANGLE_STRIP);
-      glVertex3f(wall->glseg->x1,wall->ytop,wall->glseg->z1);
-      glVertex3f(wall->glseg->x1,wall->ybottom,wall->glseg->z1);
-      glVertex3f(wall->glseg->x2,wall->ytop,wall->glseg->z2);
-      glVertex3f(wall->glseg->x2,wall->ybottom,wall->glseg->z2);
-      glEnd();
-    }
-
-    gld_EnableTexture2D(GL_TEXTURE0_ARB, true);
-    glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
-
-    if (!mlook_or_fov)
-    {
-      fV1 = SkyBox.y_offset / 127.0f;
-      fV2 = fV1 + 320.0f / 200.0f;
-    }
-    else
-    {
-      float f = viewPitch * 2 + 40 / skyscale;
-      f = BETWEEN(0, 127, f);
-      fV1 = (f + SkyBox.y_offset) / 127.0f * skyscale;
-      fV2 = fV1 + 1.0f;
-    }
-
-    k = MAX(wall->gltexture->buffer_width, 256) / 256;
-    angle = ((viewangle - ANG45) / k) % WRAPANGLE;
-
-    if (wall->flag == GLDWF_SKYFLIP)
-    {
-      fU1 = -((float)angle + SkyBox.x_offset) / (WRAPANGLE - 1);
-      fU2 = fU1 + 1.0f / k;
-    }
-    else
-    {
-      fU2 = ((float)angle + SkyBox.x_offset) / (WRAPANGLE - 1);
-      fU1 = fU2 + 1.0f / k;
-    }
-
-    glDisable(GL_DEPTH_TEST);
-    glDisable(GL_ALPHA_TEST);
-
-    glPushMatrix();
-    glMatrixMode(GL_MODELVIEW);
-    glLoadIdentity();
-
-    glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
-    gld_BindTexture(wall->gltexture, 0);
-    w = 160.0f * SCREENWIDTH / WIDE_SCREENWIDTH;
-    glBegin(GL_TRIANGLE_STRIP);
-      glTexCoord2f(fU1, fV1); glVertex3f(-w, +100.5f, -screen_skybox_zplane);
-      glTexCoord2f(fU1, fV2); glVertex3f(-w, -100.5f, -screen_skybox_zplane);
-      glTexCoord2f(fU2, fV1); glVertex3f(+w, +100.5f, -screen_skybox_zplane);
-      glTexCoord2f(fU2, fV2); glVertex3f(+w, -100.5f, -screen_skybox_zplane);
-    glEnd();
-
-    glPopMatrix();
-
-    glEnable(GL_ALPHA_TEST);
-    glEnable(GL_DEPTH_TEST);
-  }
-}
-
 // The texture offset to be applied to the texture coordinates in SkyVertex().
 static int rows, columns;
 static dboolean yflip;
@@ -544,7 +456,7 @@ void gld_GetSkyCapColors(void)
   fixedcolormap = fullcolormap;
   frame_fixedcolormap = 0;
 
-  gld_BindTexture(SkyBox.wall.gltexture, 0);
+  gld_BindSkyTexture(SkyBox.wall.gltexture);
 
   glGetTexLevelParameteriv(GL_TEXTURE_2D, 0, GL_TEXTURE_WIDTH, &width);
   glGetTexLevelParameteriv(GL_TEXTURE_2D, 0, GL_TEXTURE_HEIGHT, &height);
@@ -591,16 +503,16 @@ void gld_GetSkyCapColors(void)
 
 static void SkyVertex(vbo_vertex_t *vbo, int r, int c)
 {
-  static fixed_t scale = 10000 << FRACBITS;
-  static angle_t maxSideAngle = ANG180 / 3;
+  static const float scale = 10000.0f;
+  static const float maxSideAngle = DEG2RAD(60.0f);
 
-  angle_t topAngle= (angle_t)(c / (float)columns * ANGLE_MAX);
-  angle_t sideAngle = maxSideAngle * (rows - r) / rows;
-  fixed_t height = finesine[sideAngle>>ANGLETOFINESHIFT];
-  fixed_t realRadius = FixedMul(scale, finecosine[sideAngle>>ANGLETOFINESHIFT]);
-  fixed_t x = FixedMul(realRadius, finecosine[topAngle>>ANGLETOFINESHIFT]);
-  fixed_t y = (!yflip) ? FixedMul(scale, height) : FixedMul(scale, height) * -1;
-  fixed_t z = FixedMul(realRadius, finesine[topAngle>>ANGLETOFINESHIFT]);
+  float topAngle = DEG2RAD(c / (float)columns * 360.0f);
+  float sideAngle = maxSideAngle * (float)(rows - r) / (float)rows;
+  float height = (float)sin(sideAngle);
+  float realRadius = scale * (float)cos(sideAngle);
+  float x = realRadius * (float)cos(topAngle);
+  float y = (!yflip) ? scale * height : -scale * height;
+  float z = realRadius * (float)sin(topAngle);
   float timesRepeat;
 
   timesRepeat = (short)(4 * (256.0f / texw));
@@ -632,13 +544,13 @@ static void SkyVertex(vbo_vertex_t *vbo, int r, int c)
 
   if (r != 4)
   {
-    y += FRACUNIT * 300;
+    y += 300;
   }
 
   // And finally the vertex.
-  vbo->x =-(float)x/(float)MAP_SCALE;	// Doom mirrors the sky vertically!
-  vbo->y = (float)y/(float)MAP_SCALE + delta;
-  vbo->z = (float)z/(float)MAP_SCALE;
+  vbo->x =-x / MAP_COEFF;	// Doom mirrors the sky vertically!
+  vbo->y = y / MAP_COEFF + delta;
+  vbo->z = z / MAP_COEFF;
 }
 
 GLSkyVBO sky_vbo[2];
@@ -655,12 +567,12 @@ static void gld_BuildSky(int row_count, int col_count, SkyBoxParams_t *sky, int
   {
     Z_Free(vbo->loops);
     Z_Free(vbo->data);
-    memset(vbo, 0, sizeof(vbo[0]));
+    vbo->loops = NULL;
+    vbo->data = NULL;
   }
 
   if (!vbo->data)
   {
-    memset(vbo, 0, sizeof(vbo[0]));
     vbo->loops = Z_Malloc((row_count * 2 + 2) * sizeof(vbo->loops[0]));
     // create vertex array
     vbo->data = Z_Malloc(vertex_count * sizeof(vbo->data[0]));
@@ -737,29 +649,26 @@ static void gld_BuildSky(int row_count, int col_count, SkyBoxParams_t *sky, int
 static void RenderDome(SkyBoxParams_t *sky)
 {
   int i, j;
-  int vbosize;
   GLSkyVBO *vbo;
+  int vbosize;
+  dboolean rebuild_sky;
 
   if (!sky || !sky->wall.gltexture)
     return;
 
-  if (invul_method == INVUL_CM && frame_fixedcolormap == INVERSECOLORMAP)
+  if (invul_cm && frame_fixedcolormap == INVERSECOLORMAP)
     vbo = &sky_vbo[1];
   else
     vbo = &sky_vbo[0];
 
-  // be sure the second ARB is not enabled
-  gld_EnableDetail(false);
-
   glRotatef(-180.0f + sky->x_offset, 0.f, 1.f, 0.f);
 
   rows = 4;
   columns = 64;
-
   vbosize = 2 * rows * (columns * 2 + 2) + columns * 2;
+  rebuild_sky = sky->y_offset != y_offset_saved || sky->wall.gltexture->index != sky->index;
 
-  if (sky->y_offset != y_offset_saved ||
-      sky->wall.gltexture->index != sky->index)
+  if (rebuild_sky)
   {
     y_offset_saved = sky->y_offset;
 
@@ -771,9 +680,11 @@ static void RenderDome(SkyBoxParams_t *sky)
 
     gld_BuildSky(rows, columns, sky, 0);
     gld_BuildSky(rows, columns, sky, INVERSECOLORMAP);
+  }
 
-#ifdef USE_VBO
-    if (gl_ext_arb_vertex_buffer_object)
+  if (gl_ext_arb_vertex_buffer_object)
+  {
+    if (rebuild_sky || !vbo->id)
     {
       if (vbo->id)
       {
@@ -789,10 +700,9 @@ static void RenderDome(SkyBoxParams_t *sky)
         vbosize * sizeof(vbo->data[0]),
         vbo->data, GL_STATIC_DRAW_ARB);
     }
-#endif
   }
 
-  gld_BindTexture(SkyBox.wall.gltexture, 0);
+  gld_BindSkyTexture(SkyBox.wall.gltexture);
 
   if (gl_ext_arb_vertex_buffer_object)
   {
@@ -896,335 +806,3 @@ void gld_DrawDomeSkyBox(void)
     glShadeModel(shading_mode);
   }
 }
-
-int R_BoxSkyboxNumForName(const char *name)
-{
-  int i;
-
-  for (i = 0; i < BoxSkyboxCount; i++)
-  {
-    if (!strcasecmp(BoxSkybox[i].name, name))
-    {
-      return i;
-    }
-  }
-
-  return -1;
-}
-
-void R_SetBoxSkybox(int texture)
-{
-  int i;
-
-  BoxSkybox_default = NULL;
-
-  for (i = 0; i < BoxSkyboxCount; i++)
-  {
-    if (R_CheckTextureNumForName(BoxSkybox[i].name) == texture)
-    {
-      BoxSkybox_default = &BoxSkybox[i];
-      return;
-    }
-  }
-}
-
-box_skybox_t* R_GetBoxSkybox(int index)
-{
-  if (index >= 0 && index < BoxSkyboxCount)
-    return &BoxSkybox[index];
-  else
-    return NULL;
-}
-
-void gld_ParseSkybox(void)
-{
-  if (SC_GetString())
-  {
-    box_skybox_t sb;
-    memset(&sb, 0, sizeof(sb));
-
-    strncpy(sb.name, sc_String, 8);
-    sb.name[8] = 0;
-    M_Strupr(sb.name);
-
-    while (SC_Check())
-    {
-      SC_GetString();
-      if (SC_Compare("fliptop"))
-      {
-        sb.fliptop = true;
-      }
-    }
-
-    if (SC_GetString() && SC_Compare("{"))
-    {
-      int facecount = 0;
-
-      while (SC_GetString() && !SC_Compare("}"))
-      {
-        if (facecount < 6)
-        {
-          strcpy(sb.faces[facecount], sc_String);
-        }
-        facecount++;
-      }
-
-      if (SC_Compare("}") && (facecount == 3 || facecount == 6))
-      {
-        int i;
-        int ok = true;
-
-        for (i = 0; i < facecount; i++)
-        {
-          if (R_CheckTextureNumForName(sb.faces[i]) == -1)
-          {
-            ok = false;
-            break;
-          }
-        }
-
-        if (ok)
-        {
-          BoxSkyboxCount++;
-          BoxSkybox = Z_Realloc(BoxSkybox, BoxSkyboxCount * sizeof(BoxSkybox[0]));
-          memcpy(&BoxSkybox[BoxSkyboxCount - 1], &sb, sizeof(sb));
-        }
-      }
-    }
-  }
-
-  R_SetBoxSkybox(skytexture);
-}
-
-int gld_BindFace(box_skybox_t *sb, int index)
-{
-  int lump;
-  GLTexture *gltexture;
-  char *name = sb->faces[index];
-
-  lump = R_CheckTextureNumForName(name);
-  if (lump != -1)
-  {
-    gltexture = gld_RegisterTexture(lump, false, false, false);
-    gld_BindTexture(gltexture, GLTEXTURE_CLAMPXY);
-    return true;
-  }
-
-  return false;
-}
-
-int gld_DrawBoxSkyBox(void)
-{
-  int faces;
-  box_skybox_t *sb;
-
-  if (BoxSkyboxCount == 0)
-    return false;
-
-  if (SkyBox.side)
-  {
-    sb = R_GetBoxSkybox(SkyBox.side->skybox_index);
-  }
-  else
-  {
-    sb = BoxSkybox_default;
-  }
-
-  if (!sb)
-  {
-    return false;
-  }
-
-  gld_DrawFakeSkyStrips();
-
-  glDepthMask(false);
-
-  glDisable(GL_DEPTH_TEST);
-  glDisable(GL_ALPHA_TEST);
-
-  SetTextureMode(TM_OPAQUE);
-
-  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
-
-  glMatrixMode(GL_MODELVIEW);
-  glPushMatrix();
-
-  glLoadIdentity();
-
-  glRotatef(roll,  0.0f, 0.0f, 1.0f);
-  glRotatef(pitch, 1.0f, 0.0f, 0.0f);
-  glRotatef(yaw,   0.0f, 1.0f, 0.0f);
-  glScalef(-2.0f, 2.0f, 2.0f);
-
-  if (SkyBox.side)
-  {
-    float xoffset = (float)SkyBox.side->textureoffset * 180.0f / (float)ANG180;
-    glRotatef(-180.0f + xoffset, 0.0f, 1.0f, 0.0f);
-  }
-
-  if (sb->faces[5][0])
-  {
-    faces = 4;
-
-    // north
-    gld_BindFace(sb, 0);
-
-    glBegin(GL_TRIANGLE_FAN);
-    glTexCoord2f(0.0f, 0.0f);
-    glVertex3f(+MAXCOORD, MAXCOORD, -MAXCOORD);
-    glTexCoord2f(1.0f, 0.0f);
-    glVertex3f(-MAXCOORD, +MAXCOORD, -MAXCOORD);
-    glTexCoord2f(1.0f, 1.0f);
-    glVertex3f(-MAXCOORD, -MAXCOORD, -MAXCOORD);
-    glTexCoord2f(0.0f, 1.0f);
-    glVertex3f(+MAXCOORD, -MAXCOORD, -MAXCOORD);
-    glEnd();
-
-    // east
-    gld_BindFace(sb, 1);
-
-    glBegin(GL_TRIANGLE_FAN);
-    glTexCoord2f(0.0f, 0.0f);
-    glVertex3f(-MAXCOORD, +MAXCOORD, -MAXCOORD);
-    glTexCoord2f(1.0f, 0.0f);
-    glVertex3f(-MAXCOORD, +MAXCOORD, +MAXCOORD);
-    glTexCoord2f(1.0f, 1.0f);
-    glVertex3f(-MAXCOORD, -MAXCOORD, +MAXCOORD);
-    glTexCoord2f(0.0f, 1.0f);
-    glVertex3f(-MAXCOORD, -MAXCOORD, -MAXCOORD);
-    glEnd();
-
-    // south
-    gld_BindFace(sb, 2);
-
-    glBegin(GL_TRIANGLE_FAN);
-    glTexCoord2f(0.0f, 0.0f);
-    glVertex3f(-MAXCOORD, +MAXCOORD, +MAXCOORD);
-    glTexCoord2f(1.0f, 0.0f);
-    glVertex3f(+MAXCOORD, +MAXCOORD, +MAXCOORD);
-    glTexCoord2f(1.0f, 1.0f);
-    glVertex3f(+MAXCOORD, -MAXCOORD, +MAXCOORD);
-    glTexCoord2f(0.0f, 1.0f);
-    glVertex3f(-MAXCOORD, -MAXCOORD, +MAXCOORD);
-    glEnd();
-
-    // west
-    gld_BindFace(sb, 3);
-
-    glBegin(GL_TRIANGLE_FAN);
-    glTexCoord2f(0.0f, 0.0f);
-    glVertex3f(+MAXCOORD, +MAXCOORD, +MAXCOORD);
-    glTexCoord2f(1.0f, 0.0f);
-    glVertex3f(+MAXCOORD, +MAXCOORD, -MAXCOORD);
-    glTexCoord2f(1.0f, 1.0f);
-    glVertex3f(+MAXCOORD, -MAXCOORD, -MAXCOORD);
-    glTexCoord2f(0.0f, 1.0f);
-    glVertex3f(+MAXCOORD, -MAXCOORD, +MAXCOORD);
-    glEnd();
-  }
-  else
-  {
-    faces = 1;
-
-    // all 4 sides
-    gld_BindFace(sb, 0);
-
-    glBegin(GL_TRIANGLE_FAN);
-    glTexCoord2f(0.0f, 0.0f);
-    glVertex3f(+MAXCOORD, +MAXCOORD, -MAXCOORD);
-    glTexCoord2f(.25f, 0);
-    glVertex3f(-MAXCOORD, +MAXCOORD, -MAXCOORD);
-    glTexCoord2f(.25f, 1);
-    glVertex3f(-MAXCOORD, -MAXCOORD, -MAXCOORD);
-    glTexCoord2f(0.0f, 1.0f);
-    glVertex3f(+MAXCOORD, -MAXCOORD, -MAXCOORD);
-    glEnd();
-
-    // east
-    glBegin(GL_TRIANGLE_FAN);
-    glTexCoord2f(.25f, 0);
-    glVertex3f(-MAXCOORD, +MAXCOORD, -MAXCOORD);
-    glTexCoord2f(.5f, 0);
-    glVertex3f(-MAXCOORD, +MAXCOORD, +MAXCOORD);
-    glTexCoord2f(.5f, 1);
-    glVertex3f(-MAXCOORD, -MAXCOORD, +MAXCOORD);
-    glTexCoord2f(.25f, 1);
-    glVertex3f(-MAXCOORD, -MAXCOORD, -MAXCOORD);
-    glEnd();
-
-    // south
-    glBegin(GL_TRIANGLE_FAN);
-    glTexCoord2f(.5f, 0);
-    glVertex3f(-MAXCOORD, +MAXCOORD, +MAXCOORD);
-    glTexCoord2f(.75f, 0);
-    glVertex3f(+MAXCOORD, +MAXCOORD, +MAXCOORD);
-    glTexCoord2f(.75f, 1);
-    glVertex3f(+MAXCOORD, -MAXCOORD, +MAXCOORD);
-    glTexCoord2f(.5f, 1);
-    glVertex3f(-MAXCOORD, -MAXCOORD, +MAXCOORD);
-    glEnd();
-
-    // west
-    glBegin(GL_TRIANGLE_FAN);
-    glTexCoord2f(.75f, 0);
-    glVertex3f(+MAXCOORD, +MAXCOORD, +MAXCOORD);
-    glTexCoord2f(1.0f, 0.0f);
-    glVertex3f(+MAXCOORD, +MAXCOORD, -MAXCOORD);
-    glTexCoord2f(1.0f, 1.0f);
-    glVertex3f(+MAXCOORD, -MAXCOORD, -MAXCOORD);
-    glTexCoord2f(.75f, 1);
-    glVertex3f(+MAXCOORD, -MAXCOORD, +MAXCOORD);
-    glEnd();
-  }
-
-  // top
-  gld_BindFace(sb, faces);
-  glBegin(GL_TRIANGLE_FAN);
-  if (!sb->fliptop)
-  {
-    glTexCoord2f(0.0f, 0.0f);
-    glVertex3f(+MAXCOORD, +MAXCOORD, -MAXCOORD);
-    glTexCoord2f(1.0f, 0.0f);
-    glVertex3f(-MAXCOORD, +MAXCOORD, -MAXCOORD);
-    glTexCoord2f(1.0f, 1.0f);
-    glVertex3f(-MAXCOORD, +MAXCOORD, +MAXCOORD);
-    glTexCoord2f(0.0f, 1.0f);
-    glVertex3f(+MAXCOORD, +MAXCOORD, +MAXCOORD);
-  }
-  else
-  {
-    glTexCoord2f(0.0f, 0.0f);
-    glVertex3f(+MAXCOORD, +MAXCOORD, +MAXCOORD);
-    glTexCoord2f(1.0f, 0.0f);
-    glVertex3f(-MAXCOORD, +MAXCOORD, +MAXCOORD);
-    glTexCoord2f(1.0f, 1.0f);
-    glVertex3f(-MAXCOORD, +MAXCOORD, -MAXCOORD);
-    glTexCoord2f(0.0f, 1.0f);
-    glVertex3f(+MAXCOORD, +MAXCOORD, -MAXCOORD);
-  }
-  glEnd();
-
-  // bottom
-  gld_BindFace(sb, faces + 1);
-
-  glBegin(GL_TRIANGLE_FAN);
-  glTexCoord2f(0.0f, 0.0f);
-  glVertex3f(+MAXCOORD, -MAXCOORD, -MAXCOORD);
-  glTexCoord2f(1.0f, 0.0f);
-  glVertex3f(-MAXCOORD, -MAXCOORD, -MAXCOORD);
-  glTexCoord2f(1.0f, 1.0f);
-  glVertex3f(-MAXCOORD, -MAXCOORD, +MAXCOORD);
-  glTexCoord2f(0.0f, 1.0f);
-  glVertex3f(+MAXCOORD, -MAXCOORD, +MAXCOORD);
-  glEnd();
-
-  glPopMatrix();
-
-  glEnable(GL_ALPHA_TEST);
-  glEnable(GL_DEPTH_TEST);
-  glDepthMask(true);
-
-  SetFrameTextureMode();
-
-  return true;
-}
