@@ -1,4 +1,4 @@
-/* Emacs style mode select   -*- C++ -*-
+/* Emacs style mode select   -*- C -*-
  *-----------------------------------------------------------------------------
  *
  *
@@ -55,6 +55,8 @@
 
 #include "dsda/map_format.h"
 #include "dsda/msecnode.h"
+#include "dsda/tranmap.h"
+#include "dsda/utility.h"
 
 #define MARKED_FOR_DELETION -2
 
@@ -137,7 +139,6 @@ void P_UnArchivePlayers (void)
 
         // will be set when unarc thinker
         players[i].mo = NULL;
-        players[i].message = NULL;
         players[i].attacker = NULL;
         // HERETIC_TODO: does the rain need to be remembered?
         players[i].rain1 = NULL;
@@ -175,7 +176,22 @@ void P_ArchiveWorld (void)
     P_SAVE_X(sec->tag);
     P_SAVE_X(sec->seqType);
     P_SAVE_X(sec->flags);
+
+    // zdoom
+    P_SAVE_X(sec->gravity);
     P_SAVE_X(sec->damage);
+    P_SAVE_X(sec->lightlevel_floor);
+    P_SAVE_X(sec->lightlevel_ceiling);
+    P_SAVE_X(sec->floor_rotation);
+    P_SAVE_X(sec->ceiling_rotation);
+    P_SAVE_X(sec->floor_xscale);
+    P_SAVE_X(sec->floor_yscale);
+    P_SAVE_X(sec->ceiling_xscale);
+    P_SAVE_X(sec->ceiling_yscale);
+    P_SAVE_X(sec->floor_xoffs);
+    P_SAVE_X(sec->floor_yoffs);
+    P_SAVE_X(sec->ceiling_xoffs);
+    P_SAVE_X(sec->ceiling_yoffs);
   }
 
   for (i = 0, li = lines; i < numlines; i++, li++)
@@ -186,11 +202,12 @@ void P_ArchiveWorld (void)
     P_SAVE_X(li->special);
     P_SAVE_X(li->tag);
     P_SAVE_BYTE(li->player_activations);
-    P_SAVE_BYTE(li->arg1);
-    P_SAVE_BYTE(li->arg2);
-    P_SAVE_BYTE(li->arg3);
-    P_SAVE_BYTE(li->arg4);
-    P_SAVE_BYTE(li->arg5);
+    P_SAVE_ARRAY(li->special_args);
+
+    // zdoom
+    P_SAVE_X(li->automap_style);
+    P_SAVE_X(li->health);
+    P_SAVE_X(li->alpha);
 
     for (j = 0; j < 2; j++)
       if (li->sidenum[j] != NO_INDEX)
@@ -202,6 +219,27 @@ void P_ArchiveWorld (void)
         P_SAVE_X(si->toptexture);
         P_SAVE_X(si->bottomtexture);
         P_SAVE_X(si->midtexture);
+
+        if (map_format.zdoom)
+        {
+          P_SAVE_X(si->textureoffset_top);
+          P_SAVE_X(si->textureoffset_mid);
+          P_SAVE_X(si->textureoffset_bottom);
+          P_SAVE_X(si->rowoffset_top);
+          P_SAVE_X(si->rowoffset_mid);
+          P_SAVE_X(si->rowoffset_bottom);
+          P_SAVE_X(si->scalex_top);
+          P_SAVE_X(si->scaley_top);
+          P_SAVE_X(si->scalex_mid);
+          P_SAVE_X(si->scaley_mid);
+          P_SAVE_X(si->scalex_bottom);
+          P_SAVE_X(si->scaley_bottom);
+          P_SAVE_X(si->lightlevel);
+          P_SAVE_X(si->lightlevel_top);
+          P_SAVE_X(si->lightlevel_mid);
+          P_SAVE_X(si->lightlevel_bottom);
+          P_SAVE_X(si->flags);
+        }
       }
   }
 
@@ -230,7 +268,22 @@ void P_UnArchiveWorld (void)
     P_LOAD_X(sec->tag);
     P_LOAD_X(sec->seqType);
     P_LOAD_X(sec->flags);
+
+    // zdoom
+    P_LOAD_X(sec->gravity);
     P_LOAD_X(sec->damage);
+    P_LOAD_X(sec->lightlevel_floor);
+    P_LOAD_X(sec->lightlevel_ceiling);
+    P_LOAD_X(sec->floor_rotation);
+    P_LOAD_X(sec->ceiling_rotation);
+    P_LOAD_X(sec->floor_xscale);
+    P_LOAD_X(sec->floor_yscale);
+    P_LOAD_X(sec->ceiling_xscale);
+    P_LOAD_X(sec->ceiling_yscale);
+    P_LOAD_X(sec->floor_xoffs);
+    P_LOAD_X(sec->floor_yoffs);
+    P_LOAD_X(sec->ceiling_xoffs);
+    P_LOAD_X(sec->ceiling_yoffs);
 
     sec->ceilingdata = 0; //jff 2/22/98 now three thinker fields, not two
     sec->floordata = 0;
@@ -247,11 +300,15 @@ void P_UnArchiveWorld (void)
     P_LOAD_X(li->special);
     P_LOAD_X(li->tag);
     P_LOAD_BYTE(li->player_activations);
-    P_LOAD_BYTE(li->arg1);
-    P_LOAD_BYTE(li->arg2);
-    P_LOAD_BYTE(li->arg3);
-    P_LOAD_BYTE(li->arg4);
-    P_LOAD_BYTE(li->arg5);
+    P_LOAD_ARRAY(li->special_args);
+
+    // zdoom
+    P_LOAD_X(li->automap_style);
+    P_LOAD_X(li->health);
+    P_LOAD_X(li->alpha);
+
+    if (li->alpha < 1.f)
+      li->tranmap = dsda_TranMap(dsda_FloatToPercent(li->alpha));
 
     for (j = 0; j < 2; j++)
       if (li->sidenum[j] != NO_INDEX)
@@ -263,6 +320,28 @@ void P_UnArchiveWorld (void)
         P_LOAD_X(si->toptexture);
         P_LOAD_X(si->bottomtexture);
         P_LOAD_X(si->midtexture);
+
+        // zdoom
+        if (map_format.zdoom)
+        {
+          P_LOAD_X(si->textureoffset_top);
+          P_LOAD_X(si->textureoffset_mid);
+          P_LOAD_X(si->textureoffset_bottom);
+          P_LOAD_X(si->rowoffset_top);
+          P_LOAD_X(si->rowoffset_mid);
+          P_LOAD_X(si->rowoffset_bottom);
+          P_LOAD_X(si->scalex_top);
+          P_LOAD_X(si->scaley_top);
+          P_LOAD_X(si->scalex_mid);
+          P_LOAD_X(si->scaley_mid);
+          P_LOAD_X(si->scalex_bottom);
+          P_LOAD_X(si->scaley_bottom);
+          P_LOAD_X(si->lightlevel);
+          P_LOAD_X(si->lightlevel_top);
+          P_LOAD_X(si->lightlevel_mid);
+          P_LOAD_X(si->lightlevel_bottom);
+          P_LOAD_X(si->flags);
+        }
       }
   }
 
@@ -389,7 +468,7 @@ void P_UnArchiveMap(void)
   P_LOAD_X(automap_active);
 
   if (automap_active)
-    AM_Start();
+    AM_Start(true);
 
   P_LOAD_X(markpointnum);
 
@@ -656,7 +735,6 @@ typedef enum {
 // merges P_ArchiveThinkers & P_ArchiveSpecials
 void P_TrueArchiveThinkers(void) {
   thinker_t *th;
-  size_t    size = 0;          // killough
 
   P_SAVE_X(brain);
 
@@ -1347,6 +1425,11 @@ void P_TrueUnArchiveThinkers(void) {
             break;
           }
 
+          if (mobj->alpha < 1.f)
+            mobj->tranmap = dsda_TranMap(dsda_FloatToPercent(mobj->alpha));
+          else
+            mobj->tranmap = NULL;
+
           P_SetThingPosition (mobj);
 
           // killough 2/28/98:
