@@ -1,4 +1,4 @@
-/* Emacs style mode select   -*- C++ -*-
+/* Emacs style mode select   -*- C -*-
  *-----------------------------------------------------------------------------
  *
  *
@@ -57,6 +57,7 @@
 #include "dsda/map_format.h"
 #include "dsda/mapinfo.h"
 #include "dsda/memory.h"
+#include "dsda/music.h"
 #include "dsda/settings.h"
 #include "dsda/sfx.h"
 #include "dsda/skip.h"
@@ -269,14 +270,17 @@ void S_Start(void)
   if (muslump >= 0)
   {
     musinfo.items[0] = muslump;
-    S_ChangeMusInfoMusic(muslump, true);
+
+    if (!dsda_StartQueuedMusic())
+      S_ChangeMusInfoMusic(muslump, true);
   }
   else
   {
     memset(&musinfo, 0, sizeof(musinfo));
     musinfo.items[0] = -1;
 
-    S_ChangeMusic(mnum, true);
+    if (!dsda_StartQueuedMusic())
+      S_ChangeMusic(mnum, true);
   }
 }
 
@@ -385,6 +389,43 @@ void S_StartSoundAtVolume(void *origin_p, int sfx_id, int volume, int loop_timeo
   }
 }
 
+void S_StartSectorSound(sector_t *sector, int sfx_id)
+{
+  if (sector->flags & SECF_SILENT)
+    return;
+
+  S_StartSound((mobj_t *) &sector->soundorg, sfx_id);
+}
+
+void S_LoopSectorSound(sector_t *sector, int sfx_id, int timeout)
+{
+  if (sector->flags & SECF_SILENT)
+    return;
+
+  S_LoopSound((mobj_t *) &sector->soundorg, sfx_id, timeout);
+}
+
+void S_StartMobjSound(mobj_t *mobj, int sfx_id)
+{
+  if (mobj && mobj->subsector && mobj->subsector->sector->flags & SECF_SILENT)
+    return;
+
+  S_StartSound(mobj, sfx_id);
+}
+
+void S_StartVoidSound(int sfx_id)
+{
+  S_StartSound(NULL, sfx_id);
+}
+
+void S_StartLineSound(line_t *line, degenmobj_t *soundorg, int sfx_id)
+{
+  if (line && line->frontsector && line->frontsector->flags & SECF_SILENT)
+    return;
+
+  S_StartSound((mobj_t *) soundorg, sfx_id);
+}
+
 void S_StartSound(void *origin, int sfx_id)
 {
   S_StartSoundAtVolume(origin, sfx_id, raven ? 127 : sfx_volume, 0);
@@ -677,7 +718,6 @@ void S_StopMusic(void)
 
 void S_StopChannel(int cnum)
 {
-  int i;
   channel_t *c = &channels[cnum];
 
   if (AmbChan == cnum)
@@ -947,7 +987,7 @@ static int Raven_S_getChannel(mobj_t *origin, sfxinfo_t *sfx, sfx_params_t *para
 
   for (i = 0; i < numChannels; i++)
   {
-    if (gamestate != GS_LEVEL || origin->player)
+    if (gamestate != GS_LEVEL || origin == players[displayplayer].mo)
     {
       i = numChannels;
       break;              // let the player have more than one sound.
