@@ -1,4 +1,4 @@
-/* Emacs style mode select   -*- C++ -*-
+/* Emacs style mode select   -*- C -*-
  *-----------------------------------------------------------------------------
  *
  *
@@ -46,6 +46,7 @@
 #include "hu_stuff.h"
 
 #include "dsda/exhud.h"
+#include "dsda/font.h"
 #include "dsda/mapinfo.h"
 
 #include "heretic/in_lude.h"
@@ -426,7 +427,7 @@ static void WI_slamBackground(void)
   else if (gamemode == commercial || wbs->epsd < 0 || (gamemode == retail && wbs->epsd >= 3))
     strcpy(name, "INTERPIC");
   else
-    sprintf(name, "WIMAP%d", wbs->epsd);
+    snprintf(name, sizeof(name), "WIMAP%d", wbs->epsd);
 
   // e6y: wide-res
   V_FillBorder(-1, 0);
@@ -435,22 +436,7 @@ static void WI_slamBackground(void)
   V_DrawNamePatch(0, 0, FB, name, CR_DEFAULT, VPT_STRETCH);
 }
 
-
-// ====================================================================
-// WI_Responder
-// Purpose: Draw animations on intermission background screen
-// Args:    ev    -- event pointer, not actually used here.
-// Returns: False -- dummy routine
-//
-// The ticker is used to detect keys
-//  because of timing issues in netgames.
-dboolean WI_Responder(event_t* ev)
-{
-  return false;
-}
-
 #define SPACEWIDTH 4
-extern patchnum_t hu_font[HU_FONTSIZE];
 
 static void WI_DrawString(int cx, int cy, const char* ch)
 {
@@ -468,7 +454,7 @@ static void WI_DrawString(int cx, int cy, const char* ch)
       width += SPACEWIDTH;    // space
       continue;
     }
-    width += hu_font[c].width;
+    width += hud_font.font[c].width;
   }
   cx -= width / 2;
   if (cx < 0) cx = 0;
@@ -482,11 +468,11 @@ static void WI_DrawString(int cx, int cy, const char* ch)
       cx += SPACEWIDTH;    // space
       continue;
     }
-    w = hu_font[c].width;
+    w = hud_font.font[c].width;
     if (cx + w > 320)
       break;
 
-    V_DrawNumPatch(cx, cy, 0, hu_font[c].lumpnum, CR_GRAY, VPT_STRETCH | VPT_TRANS);
+    V_DrawNumPatch(cx, cy, 0, hud_font.font[c].lumpnum, CR_GRAY, VPT_STRETCH | VPT_TRANS);
     cx += w;
   }
 }
@@ -499,6 +485,7 @@ static void WI_DrawString(int cx, int cy, const char* ch)
 //
 const char *lf_levelname;
 const char *lf_levelpic;
+const char *lf_author;
 
 void WI_drawLF(void)
 {
@@ -511,7 +498,13 @@ void WI_drawLF(void)
   {
     // The level defines a new name but no texture for the name.
     WI_DrawString(160, y, lf_levelname);
-    y += (5 * hu_font['A' - HU_FONTSTART].height / 4);
+    y += (5 * hud_font.height / 4);
+
+    if (lf_author)
+    {
+      WI_DrawString(160, y, lf_author);
+      y += (5 * hud_font.height / 4);
+    }
   }
   else
   {
@@ -548,6 +541,7 @@ void WI_drawLF(void)
 //
 const char *el_levelname;
 const char *el_levelpic;
+const char *el_author;
 
 void WI_drawEL(void)
 {
@@ -568,6 +562,12 @@ void WI_drawEL(void)
 
     // The level defines a new name but no texture for the name.
     WI_DrawString(160, y, el_levelname);
+
+    if (el_author)
+    {
+      y += (5 * hud_font.height / 4);
+      WI_DrawString(160, y, el_author);
+    }
   }
   else
   {
@@ -1248,7 +1248,7 @@ void WI_updateDeathmatchStats(void)
     }
 
 
-    S_StartSound(0, sfx_barexp);  // bang
+    S_StartVoidSound(sfx_barexp);  // bang
     dm_state = 4;  // we're done with all 4 (or all we have to do)
   }
 
@@ -1256,7 +1256,7 @@ void WI_updateDeathmatchStats(void)
   if (dm_state == 2)
   {
     if (!(bcnt&3))
-      S_StartSound(0, sfx_pistol);  // noise while counting
+      S_StartVoidSound(sfx_pistol);  // noise while counting
 
     stillticking = false;
 
@@ -1295,7 +1295,7 @@ void WI_updateDeathmatchStats(void)
 
     if (!stillticking)
     {
-      S_StartSound(0, sfx_barexp);
+      S_StartVoidSound(sfx_barexp);
       dm_state++;
     }
   }
@@ -1303,7 +1303,7 @@ void WI_updateDeathmatchStats(void)
   {
     if (acceleratestage)
     {
-      S_StartSound(0, sfx_slop);
+      S_StartVoidSound(sfx_slop);
 
       if ( gamemode == commercial)
         WI_initNoState();
@@ -1338,11 +1338,8 @@ void WI_drawDeathmatchStats(void)
   int   y;
   int   w;
 
-  int   lh; // line height
   int   halfface = V_NamePatchWidth(facebackp)/2;
 
-  lh = WI_SPACINGY;
-
   WI_slamBackground();
 
   // draw animated background
@@ -1515,14 +1512,14 @@ void WI_updateNetgameStats(void)
       if (dofrags)
         cnt_frags[i] = WI_fragSum(i);  // we had frags
     }
-    S_StartSound(0, sfx_barexp);  // bang
+    S_StartVoidSound(sfx_barexp);  // bang
     ng_state = 10;
   }
 
   if (ng_state == 2)
   {
     if (!(bcnt&3))
-      S_StartSound(0, sfx_pistol);  // pop
+      S_StartVoidSound(sfx_pistol);  // pop
 
     stillticking = false;
 
@@ -1541,14 +1538,14 @@ void WI_updateNetgameStats(void)
 
     if (!stillticking)
     {
-      S_StartSound(0, sfx_barexp);
+      S_StartVoidSound(sfx_barexp);
       ng_state++;
     }
   }
   else if (ng_state == 4)
   {
     if (!(bcnt&3))
-      S_StartSound(0, sfx_pistol);
+      S_StartVoidSound(sfx_pistol);
 
     stillticking = false;
 
@@ -1566,14 +1563,14 @@ void WI_updateNetgameStats(void)
 
     if (!stillticking)
     {
-      S_StartSound(0, sfx_barexp);
+      S_StartVoidSound(sfx_barexp);
       ng_state++;
     }
   }
   else if (ng_state == 6)
   {
     if (!(bcnt&3))
-      S_StartSound(0, sfx_pistol);
+      S_StartVoidSound(sfx_pistol);
 
     stillticking = false;
 
@@ -1594,14 +1591,14 @@ void WI_updateNetgameStats(void)
 
     if (!stillticking)
     {
-      S_StartSound(0, sfx_barexp);
+      S_StartVoidSound(sfx_barexp);
       ng_state += 1 + 2*!dofrags;
     }
   }
   else if (ng_state == 8)
   {
     if (!(bcnt&3))
-      S_StartSound(0, sfx_pistol);
+      S_StartVoidSound(sfx_pistol);
 
     stillticking = false;
 
@@ -1620,7 +1617,7 @@ void WI_updateNetgameStats(void)
 
     if (!stillticking)
     {
-      S_StartSound(0, sfx_pldeth);
+      S_StartVoidSound(sfx_pldeth);
       ng_state++;
     }
   }
@@ -1628,7 +1625,7 @@ void WI_updateNetgameStats(void)
   {
     if (acceleratestage)
     {
-      S_StartSound(0, sfx_sgcock);
+      S_StartVoidSound(sfx_sgcock);
       if ( gamemode == commercial )
         WI_initNoState();
       else
@@ -1774,7 +1771,7 @@ void WI_updateStats(void)
     cnt_total_time = wbs->totaltimes / TICRATE;
     cnt_time = plrs[me].stime / TICRATE;
     cnt_par = wbs->partime / TICRATE;
-    S_StartSound(0, sfx_barexp);
+    S_StartVoidSound(sfx_barexp);
     sp_state = 10;
   }
 
@@ -1783,12 +1780,12 @@ void WI_updateStats(void)
     cnt_kills[0] += 2;
 
     if (!(bcnt&3))
-      S_StartSound(0, sfx_pistol);
+      S_StartVoidSound(sfx_pistol);
 
     if (cnt_kills[0] >= (plrs[me].skills * 100) / wbs->maxkills)
     {
       cnt_kills[0] = (plrs[me].skills * 100) / wbs->maxkills;
-      S_StartSound(0, sfx_barexp);
+      S_StartVoidSound(sfx_barexp);
       sp_state++;
     }
   }
@@ -1797,12 +1794,12 @@ void WI_updateStats(void)
     cnt_items[0] += 2;
 
     if (!(bcnt&3))
-      S_StartSound(0, sfx_pistol);
+      S_StartVoidSound(sfx_pistol);
 
     if (cnt_items[0] >= (plrs[me].sitems * 100) / wbs->maxitems)
     {
       cnt_items[0] = (plrs[me].sitems * 100) / wbs->maxitems;
-      S_StartSound(0, sfx_barexp);
+      S_StartVoidSound(sfx_barexp);
       sp_state++;
     }
   }
@@ -1811,7 +1808,7 @@ void WI_updateStats(void)
     cnt_secret[0] += 2;
 
     if (!(bcnt&3))
-      S_StartSound(0, sfx_pistol);
+      S_StartVoidSound(sfx_pistol);
 
     // killough 2/22/98: Make secrets = 100% if maxsecret = 0:
     if ((!wbs->maxsecret && compatibility_level < lxdoom_1_compatibility) ||
@@ -1820,14 +1817,14 @@ void WI_updateStats(void)
     {
       cnt_secret[0] = (wbs->maxsecret ?
         (plrs[me].ssecret * 100) / wbs->maxsecret : 100);
-      S_StartSound(0, sfx_barexp);
+      S_StartVoidSound(sfx_barexp);
       sp_state++;
     }
   }
   else if (sp_state == 8)
   {
     if (!(bcnt&3) && play_early_explosion) //e6y: do not play count sound after explosion sound
-      S_StartSound(0, sfx_pistol);
+      S_StartVoidSound(sfx_pistol);
 
     cnt_time += 3;
 
@@ -1853,7 +1850,7 @@ void WI_updateStats(void)
         if (compatibility_level < lxdoom_1_compatibility)
           cnt_total_time = wbs->totaltimes / TICRATE;
 
-        S_StartSound(0, sfx_barexp);
+        S_StartVoidSound(sfx_barexp);
         play_early_explosion = false; // do not play it twice or more
       }
     }
@@ -1869,7 +1866,7 @@ void WI_updateStats(void)
           cnt_total_time = wbs->totaltimes / TICRATE;
 
         if (!modifiedgame) //e6y: do not play explosion sound if it was already played
-          S_StartSound(0, sfx_barexp);
+          S_StartVoidSound(sfx_barexp);
         sp_state++;
       }
     }
@@ -1878,7 +1875,7 @@ void WI_updateStats(void)
   {
     if (acceleratestage)
     {
-      S_StartSound(0, sfx_sgcock);
+      S_StartVoidSound(sfx_sgcock);
 
       if (gamemode == commercial)
         WI_initNoState();
